/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - September 22, 2003.
   -
   - The Initial Developer of the Original Code is Henk-Johan van Rantwijk.
   - Portions created by Henk-Johan van Rantwijk are Copyright (C) 2003
   - Henk-Johan van Rantwijk.  All Rights Reserved.
   -
   - Contributor(s):
   -  Henk-Johan van Rantwijk <bugs4hj@netscape.net> (Original Author)
   -
   - Alternatively, the contents of this file may be used under the
   - terms of the GNU General Public License Version 2 or later (the
   - "GPL"), in which case the provisions of the GPL are applicable
   - instead of those above.  If you wish to allow use of your
   - version of this file only under the terms of the GPL and not to
   - allow others to use your version of this file under the MPL,
   - indicate your decision by deleting the provisions above and
   - replace them with the notice and other provisions required by
   - the GPL.  If you do not delete the provisions above, a recipient
   - may use your version of this file under either the MPL or the
   - GPL.
   -
*/

const nsIPermissionManager = Components.interfaces.nsIPermissionManager;
/*** 
/ Note: there is a max of 8 permission types defined in the mozilla source!
/ Also note that you can *only* store 256 values in a single type!
/ this sucks, big time, and should be changed a.s.a.p.
*/
var kObserverService;
var permissionmanager = null;
var popupmanager = null;
var permissionsTree;
var permissions = [];
var deletedPermissions = [];
var lastPermissionSortColumn = "";
var lastPermissionSortAscending = false;
var tabbox = null;
var newHostField = null;
var addButton = null;
// Do NOT localize this!
var permissionTypes = [ "cookie", "image", "popup" ];
var managerTypes = [ "cookieManager", "imageManager", "popupManager" ];
// But you might want to localize this
var capabilityTextID = [ "Cookies", "Images", "Popups" ];
var dialogType = "";
var typeID = 0;
var prefs = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPrefBranch);

function Startup() {
  setupIDs(window.arguments[0], true);
  tabbox = document.getElementById("tabbox");
  newHostField = document.getElementById("newHostField");
  addButton = document.getElementById("addButton");
  permissionsTree = document.getElementById("permissionsTree");

  permissionmanager = Components.classes["@mozilla.org/permissionmanager;1"].getService();
  permissionmanager = permissionmanager.QueryInterface(Components.interfaces.nsIPermissionManager);

  tabbox.selectedTab = document.getElementById(dialogType+"PermissionsTab");
  loadPermissions();
  loadTree();

  kObserverService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
  kObserverService.addObserver(permissionsReloadDisplay, "perm-changed", false);

  var hostFound = -1;

  if (window.arguments.length > 1) {
    var prefill = window.arguments[1];
    if (prefill.indexOf("www.") == 0)
      prefill = prefill.slice(4);
    hostFound = isDuplicate(prefill);
    if (hostFound >= 0) {
      permissionsTree.currentIndex = hostFound;
      permissionsTree.treeBoxObject.selection.toggleSelect(permissionsTree.currentIndex);
    }
    else {
      newHostField.value = prefill;
      hostFound = 0;
    }
  }
  updateTextField((hostFound == 0) ? true : false);
  permissionsTree.focus();
}

function setupIDs(arg, aFlag) {
  dialogType = null; // Reset selected manager

  if (aFlag) {
    for (typeID in managerTypes) {
      if (managerTypes[typeID] == arg)
        break;
    }
  }
  else {
    for (typeID in permissionTypes) {
      if (permissionTypes[typeID] == arg)
        break;
    }
  }
  dialogType = permissionTypes[typeID];
  if (!dialogType) {
    alert(mzGetTextFromBundle("unsupportedManagerSelected"));
    window.close();
  }
}

function Shutdown() {
  finalizePermissionDeletions();
  kObserverService.removeObserver(permissionsReloadDisplay, "perm-changed");
}

var permissionsReloadDisplay = {
  observe: function(subject, topic, state) {
    if (topic == "perm-changed") {
      permissions.length = 0;
      if (lastPermissionSortColumn == "rawHost") {
        lastPermissionSortAscending = !lastPermissionSortAscending;
      }
      loadPermissions();
    }
  }
}

var permissionsTreeView = {
  rowCount : 0,
  setTree : function(tree){},
  getImageSrc : function(row,column) {},
  getProgressMode : function(row,column) {},
  getCellValue : function(row,column) {},
  getCellText : function(row,column) {
    var rv="";
    if (column=="siteCol") {
      rv = permissions[row].rawHost;
    } else if (column=="statusCol") {
      rv = permissions[row].capability;
    }
    return rv;
  },
  isSeparator : function(index) {return false;},
  isSorted: function() { return false; },
  isContainer : function(index) {return false;},
  cycleHeader : function(aColId, aElt) {},
  getRowProperties : function(row,column,prop){},
  getColumnProperties : function(column,columnElement,prop){},
  getCellProperties : function(row,prop){}
 };

function Permission(number, host, rawHost, type, capability) {
  this.number = number;
  this.host = host;
  this.rawHost = rawHost;
  this.type = type;
  this.capability = capability;
}

function loadPermissions() {
  var enumerator = permissionmanager.enumerator;
  var count = 0;
  var contentStr;
  var canStr     = " are permitted from this site";
  var canSameStr = " from same host are permitted";
  var cannotStr  = " will be rejected from this site";

  canStr = capabilityTextID[typeID] + canStr;
  canSameStr = capabilityTextID[typeID] + canSameStr;
  cannotStr = capabilityTextID[typeID] + cannotStr;

  var canCannotStr = null;
  while (enumerator.hasMoreElements()) {
    var nextPermission = enumerator.getNext();
    nextPermission = nextPermission.QueryInterface(Components.interfaces.nsIPermission);
    if (nextPermission.type == dialogType) {
      var host = nextPermission.host;
      var capability = (nextPermission.capability == nsIPermissionManager.ALLOW_ACTION) ? true : false;
      canCannotStr = capability ? canStr : cannotStr;
      permissions[count] = new Permission(count++, host, (host.charAt(0)==".") ? host.substring(1,host.length) : host, nextPermission.type, canCannotStr);
    }
  }
}

function loadTree()
{
  permissionsTreeView.rowCount = permissions.length;
  permissionsTree.treeBoxObject.view = permissionsTreeView;

  if (permissions.length == 0)
    document.getElementById("removeAllPermissions").setAttribute("disabled","true");
  else 
    document.getElementById("removeAllPermissions").removeAttribute("disabled");
}

function permissionSelected() {
  var selections = GetTreeSelections(permissionsTree);
  if (selections.length) {
    document.getElementById("removePermission").removeAttribute("disabled");
  }
}

function deletePermission() {
  DeleteSelectedItemFromTree(permissionsTree, permissionsTreeView, permissions, 
                             deletedPermissions, "removePermission", "removeAllPermissions");
}

function deleteAllPermissions() {
  DeleteAllFromTree(permissionsTree, permissionsTreeView, permissions, deletedPermissions,
                   "removePermission", "removeAllPermissions");
}

function finalizePermissionDeletions() {
  var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);

  for (var p = 0; p < deletedPermissions.length; p++) {
    permissionmanager.remove(deletedPermissions[p].host, deletedPermissions[p].type);
  }
  deletedPermissions.length = 0;
}

function handlePermissionKeyPress(e) {
  if (e.keyCode == 46)
    deletePermission();
}

function permissionColumnSort(column, updateSelection) {
  lastPermissionSortAscending = SortTree(permissionsTree, permissionsTreeView, permissions, 
                                         column, lastPermissionSortColumn, lastPermissionSortAscending, 
                                         updateSelection);
  lastPermissionSortColumn = column;
}

function getLastActiveBrowserWindow()
{
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var window = windowManagerInterface.getMostRecentWindow("navigator:browser");
  return window;
}

function doHelpButton()
{
  var tabbrowser = getLastActiveBrowserWindow().getBrowser();
  tabbrowser.loadURI("http://contentblocker.mozdev.org/help/index.html?edition=premium");
  window.close();
}

function doCancelButton()
{
  if (deletedPermissions.length == 0)
    return true;
  loadPermissions();
  loadTree();
  permissionsTree.focus();
  deletedPermissions.length = 0;
  return false;
}

function togglePermission()
{
  var url = "http://"+permissions[permissionsTree.currentIndex].host;
  const gIOService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  var uri = gIOService.newURI(url, null, null);
  var capability = permissionmanager.testPermission(uri, dialogType);
  var permission = (capability == nsIPermissionManager.ALLOW_ACTION) ? nsIPermissionManager.DENY_ACTION : nsIPermissionManager.ALLOW_ACTION;
  permissionmanager.add(uri, dialogType, permission);
}

function isDuplicate(aHost)
{
  for (var i = 0; i < permissions.length; i++) {
    if (permissions[i].host == aHost)
      return i;
  }
  return -1;
}

function addPermission()
{
  var permission = nsIPermissionManager.DENY_ACTION;

  if (typeID == 2) { // popups
    permission = prefs.getBoolPref("dom.disable_open_during_load");
    // Select permission based on the pref setting (whitelisting = 1 / blacklisting = 0)
    permission = permission ? nsIPermissionManager.ALLOW_ACTION : nsIPermissionManager.DENY_ACTION;
  }
  addHostPermission(newHostField.value, permission)
}

function addHostPermission(aHost, aPermission)
{
  loadTree();
  aHost = aHost.replace(/^\s*([-\w]*:\/+)?/, ""); // trim any leading space and scheme
  var url = "http://" + aHost;
  const gIOService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  var uri = gIOService.newURI(url, null, null);
  permissionmanager.add(uri, dialogType, aPermission);
  loadTree();
}

function select(aType)
{
  dialogType = aType;
  permissions = [];
  lastPermissionSortColumn = "";
  lastPermissionSortAscending = false;
  setupIDs(tabbox.selectedTab.getAttribute("type"), false);
  loadPermissions();
  loadTree();
  updateTextField(false);
  permissionsTree.focus();
}

function updateTextField(readTextfield)
{
  if (readTextfield)
    tabbox.selectedTab.setAttribute("lastinput", newHostField.value);
  else
    newHostField.value = tabbox.selectedTab.getAttribute("lastinput") || "";

  if (newHostField.value == "")
    addButton.setAttribute("disabled", "true");
  else
    addButton.removeAttribute("disabled");
}
